@@ -148,8 +148,12 @@ def agent_cb_view(
         # handle credentials callbacks
         return handle_agent_credentials_callback(agent, topic, payload)
 
+    elif topic == TOPIC_PRESENTATIONS:
+        # handle credentials callbacks
+        return handle_agent_proof_callback(agent, topic, payload)
+
     # not yet handled message types
-    print(">>> callback:", agent.agent_name, topic)
+    print(">>> unhandled callback:", agent.agent_name, topic)
     return Response("{}")
 
 
@@ -688,6 +692,272 @@ def handle_cred_offer_response(
         return render(request, form_template, {'form': form})
 
 
+######################################################################
+# views to request, send and receive proofs
+######################################################################
+def handle_select_proof_request(
+    request,
+    form_template='aries/proof/select_request.html',
+    response_template='aries/proof/send_request.html'
+    ):
+    """
+    Select a Proof Request to send, based on the templates available in the database.
+    """
+
+    if request.method=='POST':
+        form = SelectProofRequestForm(request.POST)
+        if not form.is_valid():
+            return render(request, 'aries/form_response.html', {'msg': 'Form error', 'msg_txt': str(form.errors)})
+        else:
+            cd = form.cleaned_data
+            proof_request = cd.get('proof_request')
+            connection_id = cd.get('connection_id')
+            partner_name = cd.get('partner_name')
+
+            (agent, agent_type, agent_owner) = agent_for_current_session(request)
+
+            connections = AgentConnection.objects.filter(guid=connection_id, agent=agent).all()
+            # TODO validate connection id
+            connection = connections[0]
+
+            proof_req_attrs = proof_request.proof_req_attrs
+            proof_req_predicates = proof_request.proof_req_predicates
+
+            # selective attribute substitutions
+            institution_did = get_public_did(agent)
+            proof_req_attrs = proof_req_attrs.replace('$ISSUER_DID', institution_did)
+            proof_req_predicates = proof_req_predicates.replace('$ISSUER_DID', institution_did)
+
+            proof_form = SendProofRequestForm(initial={
+                    'agent_name': connection.agent.agent_name,
+                    'connection_id': connection_id,
+                    'partner_name': partner_name,
+                    'proof_name': proof_request.proof_req_name,
+                    'proof_attrs': proof_req_attrs,
+                    'proof_predicates': proof_req_predicates})
+
+            return render(request, response_template, {'form': proof_form})
+
+    else:
+        # find conversation request
+        (agent, agent_type, agent_owner) = agent_for_current_session(request)
+        connection_id = request.GET.get('connection_id', None)
+        connection = AgentConnection.objects.filter(guid=connection_id, agent=agent).get()
+
+        form = SelectProofRequestForm(initial={ 'connection_id': connection_id,
+                                                'partner_name': connection.partner_name,
+                                                'agent_name': connection.agent.agent_name })
+
+        return render(request, form_template, {'form': form})
+
+
+def handle_send_proof_request(
+    request,
+    template='aries/form_response.html'
+    ):
+    """
+    Send a Proof Request for the selected Proof Request.
+    User can edit the requested attributes and predicates.
+    """
+
+    if request.method=='POST':
+        form = SendProofRequestForm(request.POST)
+        if not form.is_valid():
+            return render(request, 'aries/form_response.html', {'msg': 'Form error', 'msg_txt': str(form.errors)})
+        else:
+            cd = form.cleaned_data
+            connection_id = cd.get('connection_id')
+            proof_name = cd.get('proof_name')
+            proof_attrs = cd.get('proof_attrs')
+            proof_predicates = cd.get('proof_predicates')
+
+            (agent, agent_type, agent_owner) = agent_for_current_session(request)
+    
+            connections = AgentConnection.objects.filter(guid=connection_id, agent=agent).all()
+            # TODO validate connection id
+            my_connection = connections[0]
+
+            proof_req_attrs = json.loads(proof_attrs)
+            proof_req_predicates = json.loads(proof_predicates)
+
+            requested_attrs = {}
+            for requested_attr in proof_req_attrs:
+                referent = requested_attr["name"] + "_referent"
+                requested_attrs[referent] = requested_attr
+            requested_predicates = {}
+            for requested_predicate in proof_req_predicates:
+                referent = requested_predicate["name"] + "_referent"
+                requested_predicates[referent] = requested_predicate
+
+            # build the proof request and send
+            try:
+                conversation = send_proof_request(agent, my_connection, proof_name, requested_attrs, requested_predicates)
+
+                return render(request, template, {'msg': 'Updated conversation for ' + agent.agent_name})
+            except:
+                # ignore errors for now
+                print(" >>> Failed to update conversation for", agent.agent_name)
+                return render(request, 'aries/form_response.html', {'msg': 'Failed to update conversation for ' + agent.agent_name})
+
+    else:
+        return render(request, 'aries/form_response.html', {'msg': 'Method not allowed'})
+
+
+def handle_proof_req_response(
+    request,
+    form_template='aries/proof/send_response.html',
+    response_template='aries/proof/select_claims.html'
+    ):
+    """
+    First stage in responding to a Proof Request - confirm to search for claims.
+    """
+
+    if request.method=='POST':
+        form = SendProofReqResponseForm(request.POST)
+        if not form.is_valid():
+            return render(request, 'aries/form_response.html', {'msg': 'Form error', 'msg_txt': str(form.errors)})
+        else:
+            cd = form.cleaned_data
+            conversation_id = cd.get('conversation_id')
+            proof_req_name = cd.get('proof_req_name')
+
+            (agent, agent_type, agent_owner) = agent_for_current_session(request)
+   
+            # find conversation request
+            conversations = AgentConversation.objects.filter(guid=conversation_id, connection__agent=agent).all()
+            my_conversation = conversations[0]
+            # TODO validate conversation id
+            # TODO validate connection id
+            my_connection = my_conversation.connection
+
+            # find claims for this proof request and display for the user
+            try:
+                proof_request = get_agent_conversation(agent, conversation_id, PROOF_REQ_CONVERSATION)
+                claim_data = get_claims_for_proof_request(agent, my_conversation)
+
+                form = SelectProofReqClaimsForm(initial={
+                         'conversation_id': conversation_id,
+                         'agent_name': my_connection.agent.agent_name,
+                         'from_partner_name': my_connection.partner_name,
+                         'proof_req_name': proof_req_name,
+                         'selected_claims': claim_data,
+                         'proof_request': proof_request,
+                    })
+
+                return render(request, response_template, {'form': form})
+            except Exception as e:
+                # ignore errors for now
+                print(" >>> Failed to find claims for", agent.agent_name, e)
+                return render(request, 'aries/form_response.html', {'msg': 'Failed to find claims for ' + agent.agent_name})
+
+    else:
+        # find conversation request, fill in form details
+        (agent, agent_type, agent_owner) = agent_for_current_session(request)
+        conversation_id = request.GET.get('conversation_id', None)
+        conversations = AgentConversation.objects.filter(guid=conversation_id, connection__agent=agent).all()
+        # TODO validate conversation id
+        conversation = conversations[0]
+        # TODO validate connection id
+        connection = conversation.connection
+        proof_request = get_agent_conversation(agent, conversation_id, PROOF_REQ_CONVERSATION)
+        print("proof_request:", proof_request)
+        form = SendProofReqResponseForm(initial={ 
+                                                 'conversation_id': conversation_id,
+                                                 'agent_name': agent.agent_name,
+                                                 'from_partner_name': connection.partner_name,
+                                                 'proof_req_name': proof_request['presentation_request']['name'],
+                                                })
+
+    return render(request, form_template, {'form': form})
+
+
+def handle_proof_select_claims(
+    request,
+    template='aries/form_response.html'
+    ):
+    """
+    Select claims to construct Proof for Proof Request.
+    """
+
+    if request.method=='POST':
+        form = SelectProofReqClaimsForm(request.POST)
+        if not form.is_valid():
+            return render(request, 'aries/form_response.html', {'msg': 'Form error', 'msg_txt': str(form.errors)})
+        else:
+            cd = form.cleaned_data
+            conversation_id = cd.get('conversation_id')
+            proof_req_name = cd.get('proof_req_name')
+
+            (agent, agent_type, agent_owner) = agent_for_current_session(request)
+
+            # find conversation request
+            conversations = AgentConversation.objects.filter(guid=conversation_id, connection__agent=agent).all()
+            # TODO validate conversation id
+            my_conversation = conversations[0]
+            requested_proof = get_agent_conversation(agent, conversation_id, PROOF_REQ_CONVERSATION)
+            # TODO validate connection id
+            my_connection = my_conversation.connection
+
+            # get selected attributes for proof request
+            supplied_attrs = {}
+            supplied_predicates = {}
+            supplied_self_attested_attrs = {}
+
+            # build array of credential id's (from wallet)
+            for referent in requested_proof["presentation_request"]["requested_attributes"]:
+                field_name = 'proof_req_attr_' + referent
+                value = request.POST.get(field_name)
+                if value.startswith('ref::'):
+                    supplied_attrs[referent] = { "cred_id": value[5:], "revealed": True }
+                else:
+                    supplied_self_attested_attrs[referent] = value
+            for referent in requested_proof["presentation_request"]["requested_predicates"]:
+                field_name = 'proof_req_attr_' + referent
+                value = request.POST.get(field_name)
+                if value.startswith('ref::'):
+                    supplied_predicates[referent] = { "cred_id": value[5:] }
+                else:
+                    # shouldn't happen ...
+                    supplied_predicates[referent] = { "cred_id": value }
+
+            # send claims for this proof request to requestor
+            try:
+                proof_data = send_claims_for_proof_request(agent, my_conversation, supplied_attrs, supplied_predicates, supplied_self_attested_attrs)
+
+                return render(request, template, {'msg': 'Sent proof request for ' + agent.agent_name})
+            except Exception as e:
+                # ignore errors for now
+                print(" >>> Failed to find claims for", agent.agent_name, e)
+                return render(request, 'aries/form_response.html', {'msg': 'Failed to find claims for ' + agent.agent_name})
+
+    else:
+        return render(request, 'aries/form_response.html', {'msg': 'Method not allowed'})
+
+
+def handle_view_proof(
+    request,
+    template='aries/proof/view_proof.html'
+    ):
+    """
+    View the Proof sent by the Prover.
+    """
+
+    (agent, agent_type, agent_owner) = agent_for_current_session(request)
+    conversation_id = request.GET.get('conversation_id', None)
+    conversations = AgentConversation.objects.filter(guid=conversation_id, connection__agent=agent).all()
+    # TODO validate conversation id
+    conversation = conversations[0]
+
+    requested_proof = get_agent_conversation(agent, conversation_id, PROOF_REQ_CONVERSATION)
+
+    for attr, value in requested_proof["presentation"]["requested_proof"]["revealed_attrs"].items():
+        value["identifier"] = requested_proof["presentation"]["identifiers"][value["sub_proof_index"]]
+    for attr, value in requested_proof["presentation"]["requested_proof"]["predicates"].items():
+        value["identifier"] = requested_proof["presentation"]["identifiers"][value["sub_proof_index"]]
+
+    return render(request, template, {'conversation': conversation, 'proof': requested_proof})
+
+
 ######################################################################
 # views to list wallet credentials
 ######################################################################
